﻿using ServiceCenter.Extensions;
using ServiceCenter.SqlSugars;
using System.ComponentModel;
using WCS.Core;
using WCS.Entity;
using WCS.Entity.Protocol.Station;
using WCS.WorkEngineering.Extensions;
using WCS.WorkEngineering.WebApi.Controllers;
using WCS.WorkEngineering.Worlds;
using DeviceFlags = WCS.WorkEngineering.Extensions.DeviceFlags;
using TaskStatus = WCS.Entity.TaskStatus;

namespace WCS.WorkEngineering.Systems
{
    [BelongTo(typeof(MainWorld))]
    [Description("机台叫料生成AGV任务")]
    public class 机台叫料生成AGV任务 : DeviceSystem<Device<IStation520, IStation521, IStation523>>
    {
        protected override bool ParallelDo => true;

        protected override bool SaveLogsToFile => true;

        public override void Do(Device<IStation520, IStation521, IStation523> obj)
        {
            if (!obj.Data3.Status.HasFlag(StationStatus.Auto)) return;
            if (!obj.Data3.Status.HasFlag(StationStatus.PH_Status)) return;
            if (obj.Data3.Status.HasFlag(StationStatus.PH_Status) && !obj.Data3.Status.HasFlag(StationStatus.Low_Station_2)) throw new Exception("设备有光电，但不在低位");
            if (!obj.Data3.Status.HasFlag(StationStatus.Low_Station_2)) return;

            SqlSugarHelper.Do(_db =>
            {
                var db = _db.Default;
                var task = db.Queryable<WCS_TaskInfo>().Single(x => x.ID == obj.Data.TaskNumber);
                if (task == null) throw new Exception($"未找到对应任务{obj.Data.TaskNumber}");
                if (task.Status != Entity.TaskStatus.ConveyorExecution) throw new Exception($"{task.ID}状态不是输送机执行中");

                //开始创建AGV任务
                var agvTask = db.Queryable<WCS_AgvTaskInfo>().SplitTable(x => x.Take(2)).Single(x => x.TaskId == task.ID && x.TaskType == AGVTaskType.CallMaterial);
                if (agvTask == null)
                {
                    //开始处理车间叫料AGV任务任务
                    db.InsertableRowLock(new WCS_AgvTaskInfo()
                    {
                        ID = _db.GetAgvTaskId(),
                        TaskType = AGVTaskType.CallMaterial,
                        Status = AGVTaskStatus.NewBuild,
                        TaskId = task.ID,
                        Position = task.WorkBench,
                        Station = task.SrmStation,
                        AddWho = "WCS",
                        AddTime = DateTime.Now
                    }).SplitTable().ExecuteCommand();
                    World.Log($"{task.ID}未找到对应的AGV任务");
                    return;
                }

                if (agvTask.Status != AGVTaskStatus.NewBuild) throw new Exception($"AGV任务{agvTask.ID}状态不是新建"); ;
                var agvId = DateTime.Now.GetFormat(GetFormatterEnum.only);

                var res = AgvApi.机台叫料(obj.Entity.Code, task.WorkBench, agvId);
                agvTask.AgvID = agvId;
                agvTask.Status = AGVTaskStatus.Confirm;
                agvTask.AgvStatus = AGVTaskStatus.Confirm;
                db.UpdateableRowLock(agvTask).SplitTable(x => x.Take(2)).ExecuteCommand();
                task.Status = TaskStatus.AGVExecution;
                task.AgvTaskID = agvTask.ID;
                task.EditTime = DateTime.Now;
                task.EditWho = "WCS";
                db.UpdateableRowLock(task).ExecuteCommand();
                task.AddWCS_TASK_DTL(db, obj.Entity.Code, "AGV", $"任务下发至AGV{agvId}");
            });
        }

        public override bool Select(Device dev)
        {
            return dev.HasFlag(DeviceFlags.AGV取货站台口);
        }
    }
}
